#!/bin/sh

# Makefile generator for TeX fonts.  John D. Ramsdell --- July 1990
# Extension to BDF and SNF by Carl Witty.

# Usage: $ME MFINPUT [PARAMS] [MODE] < font_list.text
# where MFINPUT is the directory which contains the METAFONT sources.
#	when PARAMS and MODE are missing, TeX font metric files are made.
#	PARAMS is a period separated set of parameters.  The first element
#              is the dots-per-inch of the device.  The rest of the 
#	       elements gives the font formats generated.  It is one of
#	       "gf", "pk", "bdf", or "snf".
#       MODE is the METAFONT mode used to generate fonts,
#   and font_list is a sequence of records in the form
#             font_name		size_1 ... size_n
#       where size is the magstep if it has no "/", otherwise it is
#	      a fraction which gives the magnification.
#
# Place this file and the font_list in the parent directory of the
# directory which is to hold the fonts.  After the first Makefile 
# has been generated, updates are are made by simply typing
# "make Makefile".

# To create fonts, type "make -k > make.log 2>&1 &" to /bin/sh, or
# "make -k >& make.log &" to /bin/csh.  If all looks fine in make.log,
# type "make clean".

MFINPUT=$1
PARAMS=$2
MODE=${3-localfont}

ME=`basename $0`
USAGE="$ME MFINPUT [PARAMS] [MODE]"

if test ! "$MFINPUT"
then
  echo Missing MFINPUT. 1>&2
  echo Usage: "$USAGE" 1>&2
  exit 1
fi

echo $ME MFINPUT PARAMS and MODE: ${MFINPUT} ${PARAMS} ${MODE} 1>&2

awk '
BEGIN {
  nparams = split("'$PARAMS'", params, ".")
  if (nparams < 2) {
    make_tfm = 1
    extension[".tfm"] = 1
  }
  else {
    dpi = int(params[1])
    for (p = 2; p <= nparams; p++)
      if ("gf" == params[p]) {
	make_gf = 1
	extension["gf"] = 1
      }
      else if ("pk" == params[p]) {
	make_pk = 1
	extension["pk"] = 1
      }
      else if ("bdf" == params[p]) {
	make_bdf = 1
	extension[".bdf"] = 1
      }
      else if ("snf" == params[p]) {
	make_snf = 1
	extension[".snf"] = 1
      }
      else 
        print "'$ME': Ignoring font parameter " params[p] | "cat 1>&2"
  }

  mode = "'$MODE'"
  base = log(1.2)
  font_data = 0
}
{
  if ($1 == "") next
  if (make_tfm) {
    basename[font_data] = $1
    font[$1] = font_data
    font_data += 1
  }
  else
    for (i = 2; i <= NF; i++) {
      basename[font_data] = $1
      if ($i == "0.0")
        scale[font_data] = 1.0
      else if ($i !~ /\//)
        scale[font_data] = exp($i*base)
      else if (split($i, fraction, "/") == 2)
        scale[font_data] = fraction[1] / fraction[2]
      else scale[font_data] = 0.0
      if (scale[font_data] > 0.0)
        font[$1 "." int(dpi*scale[font_data]+0.5)] = font_data
      else print "'$ME': Bad input record \"" $0 "\"." | "cat 1>&2"
      font_data += 1
    }
}
END {
# basename[font_data] is the font basename.
# scale[font_data] is the font scale (with 1.0 as no scaling).
# font[filename] is the font_data associated with filename.
# The use of the font array eliminates duplicates

  if (make_tfm)
    print "# Makefile for TeX Font Metrics using mode " mode "."
  else
    print "# Makefile for " dpi " lines per inch fonts using mode " mode "."
  print "# Fake targets are all and clean."
  print ""
  print "MFINPUT='$MFINPUT'"
  if (make_tfm)
    print "PARAMS=tfm"
  else {
    printf("PARAMS=%d", dpi)
    if (make_gf)
      printf("%s", ".gf")
    if (make_pk)
      printf("%s", ".pk")
    if (make_bdf)
      printf("%s", ".bdf")
    if (make_snf)
      printf("%s", ".snf")
    print ""
  }
  print "MODE=" mode
  print ""
  
  printf("all:")		# all is the default target.
  for (filename in font) 
    for (ext in extension) {
      print " \\"
      printf("  %s", filename ext)
    }
  print ""

  print ""
  print "Makefile:\t../font_list.text ../'$ME'"
  print "\t-rm -f Makefile.bak; mv Makefile Makefile.bak"
  print "\t/bin/sh ../'$ME' $(MFINPUT) $(PARAMS) $(MODE)  < ../font_list.text > Makefile"
  print ""
  print "clean:"
  if (!make_tfm) print "\t-rm -f *.tfm"
  if (!make_gf) print "\t-rm -f *gf"
  if (!make_bdf) print "\t-rm -f *.bdf"
  print "\t-rm -f *.log"
  
  for (filename in font) {	# generate all transformations.
    bn = basename[font[filename]]

    if (make_tfm) {
      print ""			# Create tfm file.
      print bn ".tfm:\t$(MFINPUT)/" bn ".mf"
      printf("\t")
      printf("mf '\''\\mode:=%s;'\'' \"batchmode;\" ", mode)
      print "input " bn ".mf"
      print "\t-rm -f " bn "*gf"
      print "\t-rm -f " bn ".log"
    }

    if (make_gf) {
      print ""			# Create gf file.
      print filename "gf:\t$(MFINPUT)/" bn ".mf"
      printf("\t")
      if (dpi <= 100) printf("-")
      printf("mf '\''\\mode:=%s;'\'' \"batchmode;\" ", mode)
      if (scale[font[filename]] != 1.0)
        printf("\"mag:=%s;\" ", scale[font[filename]])
      print "input " bn ".mf"
      print "\t-rm -f " bn ".tfm"
      print "\t-rm -f " bn ".log"
    }

    if (make_pk) {
      print ""			# Create pk file.
      if (make_gf)
        print filename "pk:\t" filename "gf"
      else {
        print filename "pk:\t$(MFINPUT)/" bn ".mf"
        printf("\t")
        if (dpi <= 100) printf("-")
        printf("mf '\''\\mode:=%s;'\'' \"batchmode;\" ", mode)
        if (scale[font[filename]] != 1.0)
          printf("\"mag:=%s;\" ", scale[font[filename]])
        print "input " bn ".mf"
        print "\t-rm -f " bn ".tfm"
        print "\t-rm -f " bn ".log"
      }
      print "\tgftopk " filename "gf"
      if (!make_gf)
	print "\t-rm -f " filename "gf"
    }

    if (make_bdf) {
      print ""			# Create bdf file.
      if (make_gf)
        print filename ".bdf:\t" filename "gf"
      else {
        print filename ".bdf:\t$(MFINPUT)/" bn ".mf"
        printf("\t")
        if (dpi <= 100) printf("-")
        printf("mf '\''\\mode:=%s;'\'' \"batchmode;\" ", mode)
        if (scale[font[filename]] != 1.0)
          printf("\"mag:=%s;\" ", scale[font[filename]])
        print "input " bn ".mf"
        print "\t-rm -f " bn ".tfm"
        print "\t-rm -f " bn ".log"
      }
      print "\tmftobdf -dpi " dpi " -dev sun " filename "gf"
      if (!make_gf)
	print "\t-rm -f " filename "gf"
    }

    if (make_snf) {
      print ""			# Create snf file.
      if (make_bdf)
        print filename ".snf:\t" filename ".bdf"
      else {
        if (make_gf)
          print filename ".snf:\t" filename "gf"
        else {
          print filename ".snf:\t$(MFINPUT)/" bn ".mf"
          printf("\t")
          if (dpi <= 100) printf("-")
          printf("mf '\''\\mode:=%s;'\'' \"batchmode;\" ", mode)
          if (scale[font[filename]] != 1.0)
            printf("\"mag:=%s;\" ", scale[font[filename]])
          print "input " bn ".mf"
          print "\t-rm -f " bn ".tfm"
          print "\t-rm -f " bn ".log"
        }
        print "\tmftobdf -dpi " dpi " -dev sun " filename "gf"
        if (!make_gf)
  	  print "\t-rm -f " filename "gf"
      }
      print "\tbdftosnf " filename ".bdf > " filename ".snf"
      if (!make_bdf)
	print "\t-rm -f " filename ".bdf"
    }
  }
}
'
